﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Linq;
using System.Threading;
using System.Windows.Controls;
using System.Windows.Media;
using Telerik.Windows.Controls.Map;

namespace DynamicItemsCollection
{
    public partial class MainPage : UserControl, INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;

        // generated items
        private ObservableCollection<DataItem> dataItems = new ObservableCollection<DataItem>();

        // visible items
        private ObservableCollection<DataItem> visibleItems = new ObservableCollection<DataItem>();
        public ObservableCollection<DataItem> VisibleItems
        {
            get
            {
                return visibleItems;
            }
            set
            {
                visibleItems = value;
                this.NotifyPropertyChanged("VisibleItems");
            }
        }

        public MainPage()
        {
            InitializeComponent();

            this.DataContext = this;

            this.GeneratePoints();

            Thread thread = new Thread(new ThreadStart(this.DynamicUpdaterThread));
            thread.IsBackground = true;
            thread.Start();
        }

        protected void NotifyPropertyChanged(string propertyName)
        {
            if (this.PropertyChanged != null)
            {
                this.PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private void GeneratePoints()
        {
            Random rnd = new Random();
            for (int i = 0; i < 200000; i++)
            {
                Location location = new Location(rnd.NextDouble() * 160 - 80, rnd.NextDouble() * 300 - 150);
                dataItems.Add(new DataItem()
                {
                    Location = location,
                    FlagColor = new SolidColorBrush(Colors.Brown)
                });
            }
        }

        private LocationRect queryRect;
        private void DynamicUpdaterThread()
        {
            while (true)
            {
                Thread.Sleep(150);

                LocationRect locationRect = this.radMap.LocationRect;
                bool update = !locationRect.IsEmpty && !locationRect.Equals(this.queryRect);
                this.queryRect = locationRect;

                if (update)
                {
                    Thread.Sleep(150);
                    if (!locationRect.Equals(this.radMap.LocationRect))
                    {
                        continue;
                    }

                    IEnumerable<DataItem> query = this.QueryItems(locationRect);
                    
                    this.Dispatcher.BeginInvoke(new Action<IEnumerable<DataItem>>((IEnumerable<DataItem> queryResult) =>
                    {
                        this.VisibleItems = new ObservableCollection<DataItem>(queryResult);
                    }), query);
                }
            }
        }

        private IEnumerable<DataItem> QueryItems(LocationRect rect)
        {
            return from DataItem shp
                         in this.dataItems
                   where ((DataItem)shp).Location.Latitude >= rect.South && ((DataItem)shp).Location.Latitude <= rect.North &&
                         ((DataItem)shp).Location.Longitude >= rect.West && ((DataItem)shp).Location.Longitude <= rect.East
                   select shp;
        }
    }
}
